@@ -18,6 +18,7 @@
             tiles: '<', // DashboardTile[]
             page: '<', // DashboardPage
             preselectedTile: '<', // DashboardTile
+            autoStackUp: '<', // boolean
             canExportDatasets: '<', // boolean
             insightsMap: '<', // { [insightId: string]: Insight }
             accessMap: '<', // { [insightId: string]: string }
@@ -35,7 +36,7 @@
             toggleGroupTileScrollBarDisplay: '&' // ({ $forceShow: boolean, $scrollTopOnHide: boolean }) => void
         },
         templateUrl: '/static/dataiku/js/dashboards/components/dashboard-grid/dashboard-grid.component.html',
-        controller: function(DashboardFilters, $timeout, $rootScope, DashboardUtils, DashboardPageUtils, TileLoadingState, TileLoadingBehavior, $element, $scope, $q, ContextualMenu, TileUtils, LinkedList, DetectUtils, Debounce) {
+        controller: function(DashboardFilters, $timeout, $rootScope, DashboardUtils, DashboardPageUtils, TileLoadingState, TileLoadingBehavior, $element, $scope, $q, ContextualMenu, TileUtils, LinkedList, DetectUtils, Debounce, WT1) {
             const ctrl = this;
 
             /*
@@ -170,6 +171,11 @@
                         syncTilesFromGridDeletionsAndMoves();
                     }, 200);
                 }
+
+                if (changes.autoStackUp && !changes.autoStackUp.isFirstChange() && ctrl.gridListRendered) {
+                    grid.toggleAutoStackUp(ctrl.autoStackUp);
+                    syncTilesFromGridDeletionsAndMoves();
+                }
             };
 
             $scope.$on('dashboardToggleTileGrouping', () => {
@@ -190,6 +196,13 @@
                 ungroupTile(tile);
             });
 
+            $scope.$on('dashboardStackUpGrid', (_, params) => {
+                if (params.grid == null || params.grid !== grid) {
+                    return;
+                }
+                stackTilesUp(params.isFromGroupTile || false);
+            });
+
             $scope.$on('dashboardEnterGroupTileEditMode', (_, { grid }) => {
                 if (ctrl.isInGroupTile || !ctrl.editable) {
                     return;
@@ -306,7 +319,22 @@
                 }
             };
 
+            ctrl.openGridMenu = function($event) {
+                if (!canStackGridTilesUp()) {
+                    return;
+                }
+
+                const menu = new ContextualMenu({
+                    template: '/templates/dashboards/grid-contextual-menu.html'
+                });
+                const newScope = $rootScope.$new();
+                newScope.stackTilesUp = () => stackTilesUp(ctrl.isInGroupTile);
+                menu.scope = newScope;
+                menu.openAtXY($event.pageX, $event.pageY);
+            };
+
             ctrl.openTileMenu = function($event, tile) {
+                $event.stopPropagation();
                 const menu = new ContextualMenu({
                     template: '/templates/dashboards/tile-contextual-menu.html'
                 });
@@ -333,6 +361,8 @@
                     newScope.deleteTile = ctrl.deleteTile;
                     newScope.handleIsTileLockedChange = ctrl.handleIsTileLockedChange;
                     newScope.ungroupTile = ungroupTile;
+                    newScope.canStackGroupTileChildrenUp = canStackGroupTileChildrenUp(tile);
+                    newScope.stackGroupTileChildrenUp = stackGroupTileChildrenUp;
                 }
 
                 menu.scope = newScope;
@@ -491,6 +521,39 @@
                 return ctrl.editable && tile.$added && ctrl.isTileSelected(tile) && ctrl.currentGroupTileIdInEditMode == null ;
             };
 
+            function stackGroupTileChildrenUp(groupTile) {
+                const tileElement = DashboardPageUtils.getTileElementFromTileId($element, groupTile.$tileId);
+                const groupTileGridElement = tileElement.find('.dashboard-grid');
+                $scope.$broadcast('dashboardStackUpGrid', { grid: groupTileGridElement.getGridListInstance(), isFromGroupTile: true });
+            }
+
+            function canStackGroupTileChildrenUp(groupTile) {
+                if (ctrl.autoStackUp || !TileUtils.isGroupTile(groupTile)) {
+                    return false;
+                }
+                const tileElement = DashboardPageUtils.getTileElementFromTileId($element, groupTile.$tileId);
+                const groupTileGridElement = tileElement.find('.dashboard-grid');
+                const groupTileGrid = groupTileGridElement.getGridListInstance();
+                return groupTileGrid != null && groupTileGrid.canStackItemsUp();
+            }
+
+            function canStackGridTilesUp() {
+                if (ctrl.autoStackUp) {
+                    return false;
+                }
+                return grid.canStackItemsUp();
+            }
+
+            function stackTilesUp(isFromGroupTile) {
+                if (isFromGroupTile) {
+                    WT1.event('dashboard-stack-up-group-tile');
+                } else {
+                    WT1.event('dashboard-stack-up-dashboard-page');
+                }
+                grid.stackItemsUp();
+                syncTilesFromGridDeletionsAndMoves();
+            }
+
             function isGroupTileEditionClick(tile, isMultiSelectKeyPressed) {
                 return !isMultiSelectKeyPressed &&
                     ctrl.isTileSelected(tile) &&
@@ -864,12 +927,12 @@
                         return Promise.resolve();
                     }
 
-                    const currentGrid = queue.shift();
+                    const currentGridElement = queue.shift();
 
                     return $timeout(() => {
-                        currentGrid.gridList('reflow');
+                        currentGridElement.getGridListInstance().reflow();
                     }).then(() => {
-                        currentGrid.find('.dashboard-grid:not(:has(.dashboard-grid))').each((_, element) => {
+                        currentGridElement.find('.dashboard-grid:not(:has(.dashboard-grid))').each((_, element) => {
                             queue.push($(element));
                         });
 
@@ -1036,6 +1099,7 @@
                             cellSizeMatchCustomHeight: ctrl.minHeight != null ? ctrl.minHeight + ctrl.tileSpacing - ctrl.padding * 2: false,
                             numberOfCellsMatchingCustomHeight: ctrl.minNumberOfRows != null ? ctrl.minNumberOfRows : false,
                             addExtraGridSpaceBelow: ctrl.addExtraGridSpaceBelow,
+                            autoStackUp: ctrl.autoStackUp,
                             offsetHeight: ctrl.isInGroupTile ? - ctrl.tileSpacing : 0,
                             // In group tiles the drop zone is the group tile container and not the group tile grid as its scrolling zone might overflow on the main grid
                             dropZone: ctrl.isInGroupTile ? DashboardPageUtils.getTileWrapperFromElement($element) : null,
