@@ -171,7 +171,7 @@
             }
         };
 
-        function filterDeployments() {
+        $scope.filterDeployments = function() {
             const filteredDeploymentStatusList = $filter('orderBy')(
                 $scope.deploymentsStatusList.filter(lightStatus => $scope.deploymentIsInUI(lightStatus)), $scope.orderByExpression
             );
@@ -180,12 +180,12 @@
 
         $scope.selectAndFilterDeployments = function(item) {
             item.$selected = !item.$selected;
-            filterDeployments();
+            $scope.filterDeployments();
         };
 
         $scope.$watch('uiState.query', (nv, ov) => {
             if (nv !== ov) {
-                filterDeployments();
+                $scope.filterDeployments();
             }
         }, true);
 
@@ -229,11 +229,15 @@
 
         $scope.clearAndRefreshFilter = function(itemList) {
             $scope.clearSelectedItems(itemList);
-            filterDeployments();
+            $scope.filterDeployments();
         }
 
-        $scope.getSelectedHealthStatuses = function() {
-            return $scope.countSelectedItems($scope.uiState.query.healthStatusMap) ? $scope.uiState.query.healthStatusMap.filter(healthStatus => healthStatus.$selected).map(healthStatus => $filter('healthStatusToFriendly')(healthStatus.id)).join(', ') : 'All states';
+        $scope.countSelectedHealthStatuses = function() {
+            return $scope.countSelectedItems($scope.uiState.query.healthStatusMap);
+        }
+
+        $scope.countSelectedTypeBadges = function() {
+            return $scope.countSelectedItems($scope.uiState.query.typeBadgesMap);
         }
 
         $scope.showOpenInANewTabMenu = function($event, url) {
@@ -379,7 +383,7 @@
         });
     });
 
-    app.controller('_DeployerDeploymentController', function($controller, $scope, $state, Dialogs, Assert, $rootScope, TaggingService, $filter, $q, $interval,
+    app.controller('_DeployerDeploymentController', function($controller, $scope, $state, Dialogs, Assert, $rootScope, TaggingService, $filter, $interval,
                                                              ActivityIndicator, WT1, SharedLastDeploymentAction) {
         $controller("_DeployerBaseController", {$scope: $scope});
         $scope.savedDeploymentSettings; // for dirtinessDetection
@@ -457,28 +461,32 @@
             }).error(setErrorInScope.bind($scope));
         };
 
+        $scope.reallyDeleteDeployment = function() {
+            WT1.event(`${$scope.deployerType}-deployer-deployment-delete`, { deploymentType: $scope.lightStatus.deploymentBasicInfo.type });
+            $scope.stopFetchLastDeploymentAction();
+            $scope.deployerAPIBase.deployments.delete($scope.lightStatus.deploymentBasicInfo.id)
+                .success(_ => $state.go(`${$scope.deployerType}deployer.deployments.dashboard`))
+                .error(function(data, status, headers, config, statusText, xhrStatus) {
+                    $scope.fetchLastDeploymentActionRepeat();
+                    // Assume a coded exception comes from the pre-delete actions
+                    if(data && data.code) {
+                        Dialogs.confirmAlert($scope, data.title, 'Do you want to delete the deployment anyway?', data.detailedMessage, "ERROR").then(function() {
+                            $scope.deployerAPIBase.deployments.delete($scope.lightStatus.deploymentBasicInfo.id, true)
+                            .success(_ => $state.go(`${$scope.deployerType}deployer.deployments.dashboard`))
+                            .error(setErrorInScope.bind($scope));
+                        });
+                    } else {
+                        setErrorInScope(data, status, headers, config, statusText, xhrStatus)
+                    }
+                });
+        }
+
         $scope.deleteDeployment = function() {
             if (!$scope.lightStatus) {
                 return;
             }
-            Dialogs.confirmAlert($scope, 'Delete deployment', 'Are you sure you want to delete this deployment?',
-                                 $scope.deleteWarning, 'WARNING').then(function() {
-                WT1.event(`${$scope.deployerType}-deployer-deployment-delete`, { deploymentType: $scope.lightStatus.deploymentBasicInfo.type });
-                $scope.deployerAPIBase.deployments.delete($scope.lightStatus.deploymentBasicInfo.id)
-                    .success(_ => $state.go(`${$scope.deployerType}deployer.deployments.dashboard`))
-                    .error(function(data, status, headers, config, statusText, xhrStatus) {
-                        // Assume a coded exception comes from the pre-delete actions
-                        if(data && data.code) {
-                            Dialogs.confirmAlert($scope, data.title, 'Do you want to delete the deployment anyway?', data.detailedMessage, "ERROR").then(function() {
-                                $scope.deployerAPIBase.deployments.delete($scope.lightStatus.deploymentBasicInfo.id, true)
-                                .success(_ => $state.go(`${$scope.deployerType}deployer.deployments.dashboard`))
-                                .error(setErrorInScope.bind($scope));
-                            });
-                        } else {
-                            setErrorInScope(data, status, headers, config, statusText, xhrStatus)
-                        }
-                    });
-            });
+
+            Dialogs.confirmAlert($scope, 'Delete deployment', 'Are you sure you want to delete this deployment?', $scope.deleteWarning, 'WARNING').then($scope.reallyDeleteDeployment);
         };
 
         $scope.lastDeploymentAction = null;
