@@ -1250,7 +1250,7 @@ app.filter('unescapeHtml', function() {
     var chars = /&(lt|gt|amp|quot|apos|#(\d+)|#x([0-9a-fA-F]+));?/g,
         esc = (function(_, code, dec, hex) {
             if (code in this) return this[code];
-            if (dec || hex) return String.fromCharCode(parseInt(dec || hex, dec ? 10 : 16));
+            if (dec || hex) return String.fromCodePoint(parseInt(dec || hex, dec ? 10 : 16));
             return _;
         }).bind({ lt: '<', gt: '>', amp: '&', quot: '"', apos: "'" });
     return function(s) { return s.replace(chars, esc); };
@@ -1421,11 +1421,24 @@ app.filter('map2Object', function() {
     };
 });
 
+app.filter('filterConfigContent', function () {
+    return function (items) {
+        if (!items) return items;
+        let filtered = {};
+        angular.forEach(items, function (value, key) {
+            if (value && key !== 'savedModels' && key !== 'llmSavedModels') { // These are handled separately with more details (ML Saved Models, Fine-tune models, ...)
+                filtered[key] = value;
+            }
+        });
+        return filtered;
+    };
+});
+
 app.filter("bundleProjectContent", function() {
     const bundleContentConfigMap = {
         datasets: 'Datasets',
         recipes: 'Recipes',
-        savedModels: 'Saved models',
+        mlSavedModels: 'ML Saved models',
         modelEvaluationStores: 'Evaluation stores',
         managedFolders: 'Managed folders',
         scenarios: 'Scenarios',
@@ -1436,7 +1449,10 @@ app.filter("bundleProjectContent", function() {
         insights: 'Insights',
         dashboards: 'Dashboards',
         knowledgeBanks: 'Knowledge Banks',
-        promptStudios: 'Prompt Studios'
+        promptStudios: 'Prompt Studios',
+        llmFineTunedModels: 'LLM Fine-tune models',
+        llmRagModels: 'LLM RAG models',
+        agents: 'Agents',
     }
     return input => bundleContentConfigMap[input];
 });
@@ -1610,7 +1626,7 @@ app.filter('prettifyScenarioType', function(ScenarioUtils) {
 
  /**
  * Displays a fallback string if the input string is null
- * 
+ *
  * @param string input containing string
  * @param string fallback string if input is null/undefined
  * @returns the string or the default value
@@ -1621,4 +1637,40 @@ app.filter('stringWithFallback', function() {
     }
 });
 
+app.filter('sortFeatureGenerationWindowOperations', function() {
+    return function(items) {
+        if (!items || !items.length) {
+            return;
+        }
+
+        const sortOrder = ["MIN", "MAX", "MEAN", "MEDIAN", "SUM", "STD"];
+
+        // Use .slice() to create a shallow copy to avoid modifying the original array
+        return items.slice().sort((a, b) => {
+            let indexA = sortOrder.indexOf(a.operation);
+            let indexB = sortOrder.indexOf(b.operation);
+
+            if (indexA === -1) {
+                indexA = Infinity;
+            }
+            if (indexB === -1) {
+                indexB = Infinity;
+            }
+
+            return indexA - indexB;
+        });
+    };
+});
+
+app.filter('pluginRecipesCategory', function() {
+     return function(plugins, recipesCategory) {
+         if (!plugins || !recipesCategory) {
+             return plugins;
+         }
+         return plugins.filter(function(plugin) {
+             return plugin.recipesCategory === recipesCategory;
+         });
+     }
+});
+
 })();
