@@ -1,4 +1,8 @@
 import logging
+import math
+import sys
+
+import numpy as np
 
 from dataiku.core import doctor_constants
 from dataiku.doctor.crossval.search_runner import TimeseriesForecastingSearchRunner
@@ -12,11 +16,13 @@ from dataiku.doctor.prediction.common import IntegerHyperparameterDimension
 from dataiku.doctor.prediction.common import OddIntegerHyperparameterDimension
 from dataiku.doctor.prediction.common import FloatHyperparameterDimension
 from dataiku.doctor.prediction.common import CategoricalHyperparameterDimension
+from dataiku.doctor.prediction.common import TreesHyperparametersSpace, get_max_features_dimension, replace_value_by_empty
 from dataiku.doctor.prediction.custom_scoring import get_custom_evaluation_metric
 from dataiku.doctor.prediction.metric import TIMESERIES_METRICS_NAME_TO_FIELD_NAME
 from dataiku.doctor.prediction.metric import CUSTOM
 from dataiku.doctor.timeseries.preparation.resampling.utils import get_frequency, get_monthly_day_alignment
 from dataiku.doctor.timeseries.utils import prefix_custom_metric_name
+
 from dataiku.core.doctor_constants import TIMESERIES_FORECAST
 from dataiku.doctor.utils.gpu_execution import get_gpu_config_from_core_params, GluonTSMXNetGPUCapability, get_default_gpu_params, GluonTSTorchGPUCapability
 
@@ -32,8 +38,10 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
     REFIT_FOR_SCORING = ModelRefit.NOT_SUPPORTED
     USE_GLUON_TS = False
     SUPPORTS_EXTERNAL_FEATURES = False
+    REQUIRES_EXTERNAL_FEATURES = False
     SUPPORTS_MODEL_COEFFICIENTS = False
     SUPPORTS_INFORMATION_CRITERIA = False
+    SUPPORTS_QUANTILES = True
     # SUPPORTS_EXTERNAL_FEATURES to be synced with:
     # platypus -> static/dataiku/js/analysis/prediction/timeseries/settings.js L.1678
     # java -> com/dataiku/dip/analysis/model/prediction/TimeseriesForecastingModelDetails.java L.43
@@ -42,16 +50,31 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
     def build(algorithm):
         return TIMESERIES_FORECASTING_ALGORITHMS_MAP[algorithm]
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         raise NotImplementedError
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         raise NotImplementedError
 
-    def get_max_used_timesteps_for_scoring(self, modeling_params, prediction_length, frequency):
-        return self.get_min_size_for_scoring(modeling_params, prediction_length)
+    def get_max_used_timesteps_for_scoring(self, modeling_params, preprocessing_params, prediction_length, frequency):
+        return self.get_min_size_for_scoring(modeling_params, preprocessing_params, prediction_length)
 
-    def get_search_runner(self, split_handler, core_params, modeling_params, model_folder_context=None):
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        """
+        Given the modeling & input params outputs a TrainableModel instance
+
+        :param input_hp_space: Input hyper-parameter space (DKU names)
+        :type input_hp_space: dict
+        :param modeling_params: Modeling params for current model
+        :type modeling_params: dict
+        :param core_params: ML task core params
+        :type core_params: dict
+        :return: trainable_model
+        :rtype: TrainableModel
+        """
+        raise NotImplementedError('Not implemented')
+
+    def get_search_runner(self, split_handler, core_params, modeling_params, preprocessing_params, model_folder_context=None):
         logger.info(
             "Create CLF from params: {} for algorithm {}".format(modeling_params, modeling_params["algorithm"])
         )
@@ -61,7 +84,7 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
 
         hyperparameter_search_params = modeling_params.get("grid_search_params", {})
         trainable_model.hyperparameters_space.set_random_state(hyperparameter_search_params.get("seed", 0))
-        
+
         search_settings = self.get_search_settings(hyperparameter_search_params, trainable_model)
 
         metrics_params = modeling_params["metrics"]
@@ -78,10 +101,12 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
             search_settings=search_settings,
             split_handler=split_handler,
             fit_before_predict=self.should_fit_before_predict(),
-            min_timeseries_size_for_training=self.get_min_size_for_training(modeling_params, core_params[doctor_constants.PREDICTION_LENGTH]),
+            min_timeseries_size_for_training=self.get_min_size_for_training(modeling_params, preprocessing_params, core_params[doctor_constants.PREDICTION_LENGTH]),
             model_folder_context=model_folder_context,
             evaluation_metric=evaluation_metric_name,
-            metric_sign=metric_sign
+            metric_sign=metric_sign,
+            skip_too_short_timeseries=len(core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS]) > 0 and core_params[doctor_constants.SKIP_TOO_SHORT_TIMESERIES_FOR_TRAINING],
+            custom_train_test_intervals=core_params[doctor_constants.CUSTOM_TRAIN_TEST_INTERVALS] if core_params.get(doctor_constants.CUSTOM_TRAIN_TEST_SPLIT) else None,
         )
 
     def should_fit_before_predict(self, force=False):
@@ -94,6 +119,234 @@ class TimeseriesForecastingAlgorithm(TabularPredictionAlgorithm):
         return False
 
 
+class TimeseriesForecastingMLAlgorithm(TimeseriesForecastingAlgorithm):
+
+    SUPPORTS_QUANTILES = False
+
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
+        shifts = preprocessing_params["feature_generation"]["shifts"]
+        windows = preprocessing_params["feature_generation"]["windows"]
+        return self.get_minimum_size_for_x_number_of_rows(shifts, windows)
+
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
+        shifts = preprocessing_params["feature_generation"]["shifts"]
+        windows = preprocessing_params["feature_generation"]["windows"]
+        return self.get_minimum_size_for_x_number_of_rows(shifts, windows, prediction_length)
+
+    @staticmethod
+    def get_minimum_size_for_x_number_of_rows(shifts, windows, prediction_length=None):
+        furthest_shift_from_horizon, furthest_shift_from_forecast, min_window_shift = sys.maxsize, sys.maxsize, sys.maxsize
+
+        if len(shifts) == 0 and len(windows) == 0:
+            return prediction_length if prediction_length else 1
+
+        if len(shifts) > 0:
+            furthest_shift_from_horizon = 1 + min([min(shift["from_horizon"]) if len(shift["from_horizon"]) > 0 else 0 for shift in shifts.values()])
+            furthest_shift_from_forecast = min([min(shift["from_forecast"]) if len(shift["from_forecast"]) > 0 else 0 for shift in shifts.values()])
+
+        if len(windows) > 0:
+            min_window_shift = min([-window["length"] + window["shift"] - (1 if window["is_from_forecast"] else 0) for window in windows])
+
+        furthest = min(furthest_shift_from_forecast, furthest_shift_from_horizon, min_window_shift)
+
+        if prediction_length:
+            furthest -= prediction_length
+
+        return abs(furthest)
+
+
+class RandomForestForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
+
+    SUPPORTS_EXTERNAL_FEATURES = True
+    REQUIRES_EXTERNAL_FEATURES = True
+
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.ml_estimator import  DkuTimeSeriesRandomForestEstimator
+        estimator = DkuTimeSeriesRandomForestEstimator(
+            frequency=get_frequency(core_params),
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+            min_samples_leaf=input_hp_space["min_samples_leaf"],
+            n_estimators=input_hp_space["n_estimators"],
+            max_features=input_hp_space["max_features"],
+            max_depth=input_hp_space["max_tree_depth"],
+            random_state=1337,
+            n_jobs=input_hp_space["n_jobs"],
+        )
+
+        # Note: `TreesHyperparametersSpace.enrich_hyperparam_point` is responsible for setting `min_samples_split`
+        hyperparameters_space = TreesHyperparametersSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class={
+                "min_samples_leaf": IntegerHyperparameterDimension,
+                "n_estimators": IntegerHyperparameterDimension
+            },
+            hp_names_to_dimension={
+                "max_features": get_max_features_dimension(input_hp_space),
+                "max_depth": IntegerHyperparameterDimension(replace_value_by_empty(input_hp_space["max_tree_depth"], value=0))
+            }
+        )
+
+        return TrainableModel(estimator, hyperparameters_space=hyperparameters_space)
+
+    def actual_params(self, ret, model, fit_params):
+        from dataiku.doctor.prediction.common import get_selection_mode
+        amp = {"resolved": ret, "other": {}}
+        params = model.get_params()
+
+        ret["rf"] = {
+            "estimators": params["n_estimators"],  # n_estimators
+            "max_tree_depth" : params["max_depth"],
+            "min_samples_leaf": params["min_samples_leaf"],
+            "selection_mode": get_selection_mode(params["max_features"]),
+        }
+
+        if ret["rf"]["selection_mode"] == "number":
+            ret["rf"]["max_features"] = params["max_features"]
+        if ret["rf"]["selection_mode"] == "prop":
+            ret["rf"]["max_feature_prop"] = params["max_features"]
+
+        amp["other"]["rf_min_samples_split"] = params["min_samples_split"]
+
+        return amp
+
+
+class RidgeForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
+
+    SUPPORTS_EXTERNAL_FEATURES = True
+    REQUIRES_EXTERNAL_FEATURES = True
+
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.ml_estimator import  DkuTimeSeriesRidgeEstimator
+        estimator = DkuTimeSeriesRidgeEstimator(
+            frequency=get_frequency(core_params),
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+            alpha_mode=input_hp_space.get("alphaMode", None),
+            alpha=input_hp_space["alpha"],
+        )
+
+        if input_hp_space.get("alphaMode", None) == "AUTO":
+            hp_space = HyperparametersSpace({})
+        else:
+            hp_space = HyperparametersSpace.from_definition(
+                input_hp_space,
+                hp_names_to_dimension_class={
+                    "alpha": FloatHyperparameterDimension
+                }
+            )
+        return TrainableModel(estimator, hyperparameters_space=hp_space)
+
+    def actual_params(self, ret, model, fit_params):
+        amp = {"resolved": ret, "other": {}}
+        params = model.get_params()
+
+        ret["ridge"] = {
+            "alpha": params["alpha"],
+        }
+
+        return amp
+
+class XGBoostForecastingAlgorithm(TimeseriesForecastingMLAlgorithm):
+
+    SUPPORTS_EXTERNAL_FEATURES = True
+    REQUIRES_EXTERNAL_FEATURES = True
+
+    def model_from_params(self, input_hp_space, modeling_params, core_params):
+        from dataiku.doctor.timeseries.models.ml_estimator import  DkuTimeSeriesXGBoostEstimator
+        from dataiku.doctor.prediction.dku_xgboost import expand_tree_method_for_xgboost
+        from dataiku.doctor.prediction.dku_xgboost import build_objective_string
+        from dataiku.doctor.prediction.common import safe_positive_int
+        from dataiku.doctor.prediction.common import create_categorical_dimension
+
+        estimator = DkuTimeSeriesXGBoostEstimator(
+            frequency=get_frequency(core_params),
+            prediction_length=core_params[doctor_constants.PREDICTION_LENGTH],
+            time_variable=core_params[doctor_constants.TIME_VARIABLE],
+            target_variable=core_params[doctor_constants.TARGET_VARIABLE],
+            timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
+            monthly_day_alignment=get_monthly_day_alignment(core_params),
+            n_estimators=input_hp_space["n_estimators"],
+            scale_pos_weight=input_hp_space['scale_pos_weight'],
+            base_score=input_hp_space['base_score'],
+            tree_method=expand_tree_method_for_xgboost(input_hp_space, get_gpu_config_from_core_params(core_params)),
+            tweedie_variance_power=input_hp_space['tweedie_variance_power'],
+            missing=input_hp_space['missing'] if input_hp_space['impute_missing'] else np.nan,
+            random_state=1337,
+            n_jobs=safe_positive_int(input_hp_space['nthread']),
+        )
+
+        booster = CategoricalHyperparameterDimension(input_hp_space["booster"])
+        if len(booster._get_enabled_values_list()) == 0:
+            booster = create_categorical_dimension(["gbtree"])
+        objective = CategoricalHyperparameterDimension(input_hp_space["objective"])
+        enabled_val_list = [build_objective_string(val) for val in objective._get_enabled_values_list()]
+        enabled_val_dict = create_categorical_dimension(enabled_val_list)._get_values()
+        objective.set_values(enabled_val_dict)
+        if len(objective._get_enabled_values_list()) == 0:
+            objective = create_categorical_dimension([build_objective_string("reg_linear")])
+
+        hyperparameters_space = HyperparametersSpace.from_definition(
+            input_hp_space,
+            hp_names_to_dimension_class={
+                "max_depth": IntegerHyperparameterDimension,
+                "learning_rate": FloatHyperparameterDimension,
+                "gamma": FloatHyperparameterDimension,
+                "min_child_weight": FloatHyperparameterDimension,
+                "max_delta_step": FloatHyperparameterDimension,
+                "subsample": FloatHyperparameterDimension,
+                "colsample_bytree": FloatHyperparameterDimension,
+                "colsample_bylevel": FloatHyperparameterDimension
+            },
+            hp_names_to_dimension={
+                "reg_alpha": FloatHyperparameterDimension(input_hp_space['alpha']),
+                "reg_lambda": FloatHyperparameterDimension(input_hp_space['lambda']),
+                "booster": booster,
+                "objective": objective
+            }
+        )
+
+        return TrainableModel(estimator, hyperparameters_space=hyperparameters_space)
+
+    def actual_params(self, ret, model, fit_params):
+
+        params = model.get_params()
+        missing = None if np.isnan(params["missing"]) else params["missing"]
+        amp = {"resolved": ret, "other": {}}
+        ret["xgboost"] = {
+            "max_depth": params["max_depth"],
+            "learning_rate": params["learning_rate"],
+            "n_estimators": params["n_estimators"],
+            "nthread": params["n_jobs"] if params["n_jobs"] > 0 else -1,
+            "gamma": params["gamma"],
+            "min_child_weight": params["min_child_weight"],
+            "max_delta_step": params["max_delta_step"],
+            "subsample": params["subsample"],
+            "colsample_bytree": params["colsample_bytree"],
+            "colsample_bylevel": params["colsample_bylevel"],
+            "alpha": params["reg_alpha"],
+            "lambda": params["reg_lambda"],
+            "seed": params["random_state"],
+            "impute_missing": True if missing is not None else False,
+            "missing": missing,
+            "base_score": params["base_score"],
+            "scale_pos_weight": params["scale_pos_weight"],
+            # TODO: sc-258536 support XGBoost early stopping for time series forecasting
+            # "enable_early_stopping": fit_params.get('early_stopping_rounds') is not None,
+            # "early_stopping_rounds": fit_params.get('early_stopping_rounds'),
+            "booster": params.get("booster"),
+            "objective": params.get("objective").replace(":", "_"),
+            "tweedie_variance_power": params.get("tweedie_variance_power")
+        }
+        return amp
+
+
 class GluonTSTrivialIdentity(TimeseriesForecastingAlgorithm):
     USE_GLUON_TS = True
 
@@ -117,10 +370,10 @@ class GluonTSTrivialIdentity(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return prediction_length
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         return prediction_length
 
 
@@ -153,10 +406,10 @@ class GluonTSSeasonalNaive(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return max(prediction_length, modeling_params["seasonal_naive_timeseries_params"]["season_length"])
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_season_length = get_parameter_max_value(modeling_params, "season_length")
         return max(prediction_length, max_season_length)
 
@@ -210,13 +463,13 @@ class GluonTSNPTSForecaster(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return max(prediction_length, 10)  # 10 is a "reasonable" arbitrary value
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         return max(prediction_length, 10)  # 10 is a "reasonable" arbitrary value
 
-    def get_max_used_timesteps_for_scoring(self, modeling_params, prediction_length, frequency):
+    def get_max_used_timesteps_for_scoring(self, modeling_params, preprocessing_params, prediction_length, frequency):
         return get_used_context_length(modeling_params, "gluonts_npts_timeseries_params", 1100)
 
 
@@ -288,15 +541,17 @@ class GluonTSTorchSimpleFeedForward(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return get_used_context_length(modeling_params, "gluonts_torch_simple_feedforward_timeseries_params", prediction_length)
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_context_length = get_max_context_length(modeling_params, prediction_length)
         return max_context_length + prediction_length
 
+
 class GluonTSTorchDeepAR(TimeseriesForecastingAlgorithm):
     USE_GLUON_TS = True
+    SUPPORTS_EXTERNAL_FEATURES = True
 
     def model_from_params(self, input_hp_space, modeling_params, core_params):
         gpu_config = get_gpu_config_from_core_params(core_params)
@@ -316,7 +571,7 @@ class GluonTSTorchDeepAR(TimeseriesForecastingAlgorithm):
             time_variable=core_params[doctor_constants.TIME_VARIABLE],
             target_variable=core_params[doctor_constants.TARGET_VARIABLE],
             timeseries_identifiers=core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS],
-            use_timeseries_identifiers_as_features=input_hp_space.get("use_timeseries_identifiers_as_features", False),
+            use_timeseries_identifiers_as_features=False,
             full_context=input_hp_space["full_context"],
             scaling=input_hp_space["scaling"],
             num_parallel_samples=input_hp_space["num_parallel_samples"],
@@ -373,17 +628,18 @@ class GluonTSTorchDeepAR(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return get_used_context_length(modeling_params, "gluonts_torch_deepar_timeseries_params", prediction_length)
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_context_length = get_max_context_length(modeling_params, prediction_length)
         return max_context_length + prediction_length
 
-    def get_max_used_timesteps_for_scoring(self, modeling_params, prediction_length, frequency):
+    def get_max_used_timesteps_for_scoring(self, modeling_params, preprocessing_params, prediction_length, frequency):
         # importing here so that forecasting ML task won't always fail if gluonts is not installed
         from gluonts.time_feature import get_lags_for_frequency
-        return self.get_min_size_for_scoring(modeling_params, prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
+        return self.get_min_size_for_scoring(modeling_params, preprocessing_params,
+                                             prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
 
 
 class GluonTSSimpleFeedForward(TimeseriesForecastingAlgorithm):
@@ -449,10 +705,10 @@ class GluonTSSimpleFeedForward(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return get_used_context_length(modeling_params, "gluonts_simple_feedforward_timeseries_params", prediction_length)
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_context_length = get_max_context_length(modeling_params, prediction_length)
         return max_context_length + prediction_length
 
@@ -536,17 +792,18 @@ class GluonTSDeepAR(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return get_used_context_length(modeling_params, "gluonts_deepar_timeseries_params", prediction_length)
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_context_length = get_max_context_length(modeling_params, prediction_length)
         return max_context_length + prediction_length
 
-    def get_max_used_timesteps_for_scoring(self, modeling_params, prediction_length, frequency):
+    def get_max_used_timesteps_for_scoring(self, modeling_params, preprocessing_params, prediction_length, frequency):
         # importing here so that forecasting ML task won't always fail if gluonts is not installed
         from gluonts.time_feature import get_lags_for_frequency
-        return self.get_min_size_for_scoring(modeling_params, prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
+        return self.get_min_size_for_scoring(modeling_params, preprocessing_params,
+                                             prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
         
 
 class GluonTSTransformer(TimeseriesForecastingAlgorithm):
@@ -619,17 +876,18 @@ class GluonTSTransformer(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return get_used_context_length(modeling_params, "gluonts_transformer_timeseries_params", prediction_length)
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_context_length = get_max_context_length(modeling_params, prediction_length)
         return max_context_length + prediction_length
 
-    def get_max_used_timesteps_for_scoring(self, modeling_params, prediction_length, frequency):
+    def get_max_used_timesteps_for_scoring(self, modeling_params, preprocessing_params, prediction_length, frequency):
         # importing here so that forecasting ML task won't always fail if gluonts is not installed
         from gluonts.time_feature import get_lags_for_frequency
-        return self.get_min_size_for_scoring(modeling_params, prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
+        return self.get_min_size_for_scoring(modeling_params, preprocessing_params,
+                                             prediction_length) + max(get_lags_for_frequency(freq_str=frequency))
 
 
 class GluonTSMQCNN(TimeseriesForecastingAlgorithm):
@@ -701,10 +959,10 @@ class GluonTSMQCNN(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return get_used_context_length(modeling_params, "gluonts_mqcnn_timeseries_params", 4 * prediction_length)
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_context_length = get_max_context_length(modeling_params, 4 * prediction_length)
         return max_context_length + prediction_length
 
@@ -781,10 +1039,10 @@ class AutoArima(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return prediction_length + max(prediction_length, modeling_params["auto_arima_timeseries_params"]["m"])
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_season_length = get_parameter_max_value(modeling_params, "m")
         return 2 * prediction_length + 2 * max_season_length
 
@@ -839,10 +1097,10 @@ class Arima(TimeseriesForecastingAlgorithm):
         return amp
 
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return prediction_length + max(prediction_length, modeling_params["arima_timeseries_params"]["s"])
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         s = modeling_params['arima_grid']['s']
         return 2 * prediction_length + 2 * s
 
@@ -906,10 +1164,10 @@ class SeasonalTrendLoess(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return max(prediction_length, modeling_params["seasonal_loess_timeseries_params"]["period"], 10)  # STL requires at least 10 time steps for training
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         max_season_length = get_parameter_max_value(modeling_params, "period")
         return max(prediction_length, max_season_length, 10)  # STL requires at least 10 time steps for training
 
@@ -961,10 +1219,10 @@ class ETS(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         return max(prediction_length, modeling_params["ets_params"]["seasonal_periods"])
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         # Heuristic method needs at least 10 observations
         return max(prediction_length, modeling_params["ets_timeseries_grid"]["seasonal_periods"], 10)
 
@@ -1030,11 +1288,11 @@ class Prophet(TimeseriesForecastingAlgorithm):
         amp = {"resolved": ret}
         return amp
 
-    def get_min_size_for_scoring(self, modeling_params, prediction_length):
+    def get_min_size_for_scoring(self, modeling_params, preprocessing_params, prediction_length):
         # even though Prophet.predict works with only 1 past value, we use 2 in case we re-fit in the scoring/evaluation recipe
         return 2
 
-    def get_min_size_for_training(self, modeling_params, prediction_length):
+    def get_min_size_for_training(self, modeling_params, preprocessing_params, prediction_length):
         # 2 is the minimum number of past values required in the Prophet.fit method
         return 2
 
@@ -1084,4 +1342,8 @@ TIMESERIES_FORECASTING_ALGORITHMS_MAP = {
     "ETS": ETS(),
     "SEASONAL_LOESS": SeasonalTrendLoess(),
     "PROPHET": Prophet(),
+    # ML-based forecasting algorithms
+    "RANDOM_FOREST_REGRESSION": RandomForestForecastingAlgorithm(),
+    "XGBOOST_REGRESSION": XGBoostForecastingAlgorithm(),
+    "RIDGE_REGRESSION": RidgeForecastingAlgorithm(),
 }
