@@ -56,6 +56,7 @@ class RAGServer:
     print_sources: bool
     output_format: str
     filter: FilterDesc
+    run_counter: int = 1
 
     def __init__(self) -> None:
         self.started = False
@@ -120,9 +121,10 @@ class RAGServer:
 
         self.filter = start_command.get("filter")
 
-        self.dku_vector_store = VectorStoreFactory.get_vector_store(kb, isolated_folder.folder_path)
+        self.dku_vector_store = VectorStoreFactory.get_vector_store(kb, isolated_folder.folder_path, VectorStoreFactory.get_connection_details_from_env)
         self.started = True
 
+
     def process_prompt(self, process_prompt_command: ProcessSinglePromptCommand) -> SimpleCompletionResponseOrError:
         assert self.started, "Not started"
 
@@ -357,13 +359,15 @@ class RAGServer:
         with self.lock:
             with trace.subspan("RAG_RETRIEVE_DOCUMENTS") as retrieve_span:
                 additional_search_kwargs: Dict = {}
-
                 if self.rag_settings.get("enforceDocumentLevelSecurity"):
                     caller_security_tokens = self._extract_caller_security_tokens(command)
                     self.dku_vector_store.add_security_filter(additional_search_kwargs, caller_security_tokens)
                 if self.filter is not None:
                     self.dku_vector_store.add_filter(additional_search_kwargs, self.filter)
 
+                if callerFilters := command["query"].get("context", {}).get("callerFilters"):
+                    self.dku_vector_store.add_dynamic_filter(additional_search_kwargs, callerFilters,  None)
+
                 retriever = self.dku_vector_store.as_retriever(self.embeddings, self.rag_settings, additional_search_kwargs)
                 docs = retriever.invoke(question)
 
@@ -566,16 +570,21 @@ class RAGServer:
             )
         elif command["type"] == "process-completion-query":
             logger.debug("Received command: %s", command)
-            if command["stream"]:
-                async for c in self.process_prompt_stream(command):
-                    if isinstance(c, DSSLLMStreamedCompletionChunk):
-                        yield {"chunk" : c.data}
-                    else:
-                        yield {"footer" : c.data}
-            else:
-                yield await asyncio.get_running_loop().run_in_executor(
-                    self.executor, self.process_prompt, command
-                )
+            logger.info("\n===============  Start completion query - run %s ===============", self.run_counter)
+            try:
+                if command["stream"]:
+                    async for c in self.process_prompt_stream(command):
+                        if isinstance(c, DSSLLMStreamedCompletionChunk):
+                            yield {"chunk" : c.data}
+                        else:
+                            yield {"footer" : c.data}
+                else:
+                    yield await asyncio.get_running_loop().run_in_executor(
+                        self.executor, self.process_prompt, command
+                    )
+                logger.info("\n===============  End completion query - run %s ===============", self.run_counter)
+            finally:
+                self.run_counter += 1
         else:
             raise Exception("Unknown command type: %s" % command["type"])
 
